####SimpleFirewall PM functions Luis Wong <lwong@mpsnet.com.mx>

package SimpleFirewall;

use XML::Parser;

sub new {
	my $this ={};
	#$this->{NOME} = undef;
	$this->{fw} = ();
	$this->{fwItems} = ();
	$this->{fw_file} = '';
	$this->{fwservices_file} = '';
	bless $this;
	return $this
}

##
# Public method for get firewall info (after LoadFirewall)
sub GetZoneList {
	my $this = shift;
	return sort( keys %{ $this->{fw}{ZONE} } );
}
sub GetNetList {
	my $this = shift;
	return sort( keys %{ $this->{fw}{NET} } );
}
sub GetHostList {
	my $this = shift;
	return sort( keys %{ $this->{fw}{HOST} } );
}
sub GetGroupList {
	my $this = shift;
	return sort( keys %{ $this->{fw}{GROUP} } );
}
sub GetZone {
	my ($this,$name) = @_;
	return %{ $this->{fw}{ZONE}{$name} };
}
sub GetNet {
	my ($this,$name) = @_;
	return %{ $this->{fw}{NET}{$name} };
}
sub GetHost {
	my ($this,$name) = @_;
	return %{ $this->{fw}{HOST}{$name} };
}
sub GetGroup {
	my ($this,$name) = @_;
	return sort( @{ $this->{fw}{GROUP}{$name}} );
}

sub GetRedir {
    my ($this,$n) = @_;
    return %{ $this->{fw}{REDIR}[$n-1] };
}

sub GetRedirsCount {
    my $this = shift;
    return $#{$this->{fw}{REDIR}}+1;
}


sub GetAllItemsList {
	my $this = shift;
	return sort( keys %{ $this->{fwItems} } );
}
sub GetServicesList {
	my $this = shift;
	return sort( keys %{ $this->{services} } );
}
sub GetService {
	my $this = shift;
	my $name = shift;
	return %{ $this->{services}{$name} };
}

sub GetMasqueradesCount {
	my $this = shift;
	return $#{$this->{fw}{MASQUERADE}}+1;
}
sub GetMasquerade {
	# param n = id of masquerade rule (1 .. MasqueradeCount)
	my ($this,$n) = @_;
	return %{ $this->{fw}{MASQUERADE}[$n-1] };
}

sub GetNatsCount {
	my $this = shift;
	return $#{$this->{fw}{NAT}}+1;
}
sub GetNat {
	my ($this,$n) = @_;
	return %{ $this->{fw}{NAT}[$n-1] };
}

sub GetRulesCount {
	my $this = shift;
	return $#{$this->{fw}{RULE}}+1;
}



sub GetRule {
	my ($this,$n) = @_;
	return %{ $this->{fw}{RULE}[$n-1] };
}


# AddGroup( $group, @items );
sub AddGroup {
	my $this = shift;
	my $group = shift;
	my @items = @_;
	@{ $this->{fw}{GROUP}{$group} } = @items;
	$this->{fwItems}{$group} = 'GROUP';
	return 1;
}
# AddHost( $name, $ip, $zone )
sub AddHost {
	my ($this, $name, $ip, $zone) = @_;
	%{ $this->{fw}{HOST}{$name} } = ('NAME'=>$name, 'IP'=>$ip, 'ZONE'=>$zone );
	$this->{fwItems}{$name} = 'HOST';
}


sub AddRedir {
    my ($this,$idx,$name, $s_ip, $s_port, $p_ip, $p_port) = @_;

#    %{ $this->{fw}{REDIR}{$name} } = ('NAME'=>$name, 'S_IP'=>$s_ip,'S_PORT'=>$s_port, 'P_IP'=>$p_ip,'P_PORT'=>$p_port );
   # $this->{fwItems}{$name} = 'REDIR';
    my %attr =  ('NAME'=>$name, 'S_IP'=>$s_ip,'S_PORT'=>$s_port, 'P_IP'=>$p_ip,'P_PORT'=>$p_port );
    if( $idx == 0 ) {
	%{$this->{fw}{'REDIR'}[$#{$this->{fw}{'REDIR'}}+1]} = %attr;
    } else {
	%{$this->{fw}{'REDIR'}[$idx-1]} = %attr;
    }
 }

 # AddNet( $name, $ip, $netmask, $zone )
sub AddNet {
	my ($this, $name, $ip, $netmask, $zone) = @_;
	%{ $this->{fw}{NET}{$name} } = ('NAME'=>$name, 'IP'=>$ip, 'NETMASK'=>$netmask,'ZONE'=>$zone );
	$this->{fwItems}{$name} = 'NET';
	return 1;
}
# AddZone( $name, $if )
sub AddZone {
	my ($this, $name, $if) = @_;
	%{ $this->{fw}{ZONE}{$name} } = ('NAME'=>$name, 'IF'=>$if );
	$this->{fwItems}{$name} = 'ZONE';
	return 1; 
}
# AddMasquerade( $idx, $zone ) if $idx==0 then add new Masquerade
sub AddMasquerade {
	my ($this, $idx, $zone) = @_;
	if( $idx == 0 ) {
		%{ $this->{fw}{MASQUERADE}[$#{$this->{fw}{MASQUERADE}}+1] } = ('ZONE'=>$zone);
	} else {
		%{ $this->{fw}{MASQUERADE}[$idx-1] } = ('ZONE'=>$zone);
	}
}
# AddNat( $idx, $virtual, $real ) if $idx==0 then add new Masquerade
sub AddNat {
	my ($this, $idx, $virtual, $real) = @_;
	if( $idx == 0 ) {
		%{ $this->{fw}{'NAT'}[$#{$fw{'NAT'}}+1] } = ('VIRTUAL'=>$virtual, 'REAL'=>$real);
	} else {
		%{ $this->{fw}{NAT}[$idx-1] } = ('VIRTUAL'=>$virtual, 'REAL'=>$real);
	}
}
# AddRule( $idx, $src, $dst, $service, $port, $active );
sub AddRule {
	my ($this, $idx, $src, $dst, $service, $port, $active ) = @_;
	my %attr = ( 'SRC'=>$src, 'DST'=>$dst, 'SERVICE'=>$service);
	if( $port ne '' ) { $attr{PORT} = $port; }
	if( ! $active or $active eq 0) { $attr{ACTIVE} = 'NO'; }
	if( $idx == 0 ) {
		%{$this->{fw}{'RULE'}[$#{$this->{fw}{'RULE'}}+1]} = %attr;
	} else {
		%{$this->{fw}{'RULE'}[$idx-1]} = %attr;
	}
}


# DeleteItem( $name );
sub DeleteItem {
	my $this = shift;
	my $name = shift;
	my $type = $this->{fwItems}{$name};
	delete $this->{fw}{$type}{$name};
	delete $this->{fwItems}{$name};
}
# DeleteGroup( $group );
sub DeleteGroup {
	my ($this, $group) = @_;
	$this->DeleteItem( $group );
}
# DeleteHost( $host );
sub DeleteHost {
	my ($this, $host) = @_;
	$this->DeleteItem( $host );
}
# DeleteHost( $net );
sub DeleteNet {
	my ($this, $net) = @_;
	$this->DeleteItem( $net );
}
# DeleteZone( $zone );
sub DeleteZone {
	my ($this, $zone) = @_;
	$this->DeleteItem( $zone );
}

sub DeleteRedir {
    my ($this, $idx) = @_;
	splice( @{$this->{fw}{'REDIR'}}, $idx-1, 1 );
	
}


# DeleteMasquerade( $idx );
sub DeleteMasquerade {
	my ($this, $idx) = @_;
	splice( @{$this->{fw}{'MASQUERADE'}}, $idx-1, 1 );
}
# DeleteNat( $idx );
sub DeleteNat {
	my ($this, $idx) = @_;
	splice( @{$this->{fw}{'NAT'}}, $idx-1, 1 );
}
# DeleteRule( $idx );
sub DeleteRule {
	my ($this, $idx) = @_;
	splice( @{$this->{fw}{'RULE'}}, $idx-1, 1 );
}


sub LoadFirewall {
	my $this = shift;
	my $fwFile = shift;

	$this->{fw_file} = $fwFile;


	$this->{fwItems}{FIREWALL} = 'ZONE';
	%{$this->{fw}{ZONE}{FIREWALL}} = (NAME=>'FIREWALL');

	my $xml = new XML::Parser( Style=>'Tree' );
	my @tree = @{ $xml->parsefile( $fwFile ) };


	for( my $i=0; $i<=$#tree; $i+=2 ) {
		my $name = uc($tree[$i]);
		if ($name eq 'FIREWALL') {
			my @list = @{$tree[$i+1]};
			my %attr = shift @list;

			

			for( my $j=0; $j<=$#list; $j+=2 ) {
			    my $name2 = uc($list[$j]);
			    if( $name2 eq 'ZONE' ) { $this->_LoadFirewallItem( 'ZONE', @{$list[$j+1]} ); }
			    if( $name2 eq 'REDIR' ) { $this->_LoadFirewallRedir( 'REDIR', @{$list[$j+1]} ); }
			    if( $name2 eq 'NET' ) { $this->_LoadFirewallItem( 'NET', @{$list[$j+1]} ); }
			    if( $name2 eq 'HOST' ) { $this->_LoadFirewallItem( 'HOST', @{$list[$j+1]} ); }
			    if( $name2 eq 'GROUP' ) { $this->_LoadFirewallItem( 'GROUP', @{$list[$j+1]} ); }
			    if( $name2 eq 'MASQUERADE' ) { $this->_LoadFirewallNat( 'MASQUERADE', @{$list[$j+1]} ); }
			    if( $name2 eq 'NAT' ) { $this->_LoadFirewallNat( 'NAT', @{$list[$j+1]} ); }
			    if( $name2 eq 'RULE' ) { $this->_LoadFirewallRule( @{$list[$j+1]} ); }
			}
		}
	}
}

sub _LoadFirewallItem {
	my $this = shift;

	my $type = shift;
	my @list = @_;

	my %attrs = upperKeys( %{shift @list} );
	my $name = $attrs{'NAME'};

	if( $this->{fwItems}{$name} ne '' ) {
		die( "Error:\"".$name."\"  present.\n" );
	}
	$this->{fwItems}{$name} = $type;
	if( $type eq 'GROUP' ) {
		$this->{fw}{'GROUP'}{$name} = ();
		for( my $j=0; $j<=$#list; $j+=2 ) {
			if( $list[$j] ne '0' ) {
				my %item_attrs = upperKeys( %{ shift @{$list[$j+1]} } );
				my $item = $item_attrs{'NAME'};
				if( $this->{fwItems}{$item} ne '' ) {
					push @{$this->{fw}{'GROUP'}{$name}}, $item;
				} else {
					die( "Error: item $item  group $name not defined.\n" );
				}
			}
		}
	} else {
		%{$this->{fw}{$type}{$name}} = %attrs;
	}
}

sub _LoadFirewallRedir {
	my $this = shift;
	my $type = shift;
	my @list = @_;

	my %attrs = upperKeys( %{shift @list} );
	%{$this->{fw}{$type}[$#{$this->{fw}{$type}}+1]} = %attrs;
}

###
# Internal method for add NAT or MASQUERADE to firewall object
sub _LoadFirewallNat {
	my $this = shift;
	my $type = shift;
	my @list = @_;
	#my $name = $list[$i];
	my %attrs = upperKeys( %{shift @list} );
	%{$this->{fw}{$type}[$#{$this->{fw}{$type}}+1]} = %attrs;
}

###
# Internal method for add RULE to firewall object
sub _LoadFirewallRule {
	my $this = shift;
	my @list = @_;
	my %attrs = upperKeys( %{shift @list} );
	if( $this->{fwItems}{$attrs{'SRC'}} eq '' ) {
		die( "Error: La rule ".($#{$fw{RULE}}+2)." Origin not valid.\n" );
	}
	if( $this->{fwItems}{$attrs{'DST'}} eq '' ) {
		die( "Error: La rule ".($#{$fw{RULE}}+2)." Dest not valid.\n" );
	}
	%{$this->{fw}{'RULE'}[$#{$this->{fw}{'RULE'}}+1]} = %attrs;
}




sub LoadServices {
	my $this = shift;
	my $servicesFile = shift;

	$this->{fwservices_file} = $servicesFile;

	my $xml = new XML::Parser( Style=>'Tree' );
	my @tree = @{ $xml->parsefile( $servicesFile ) };


	for( my $i=0; $i<=$#tree; $i+=2 ) {
		my $name = uc($tree[$i]);
		if ($name eq 'SERVICES') {
			my @list = @{$tree[$i+1]};
			shift @list;


			for( my $j=0; $j<=$#list; $j+=2 ) {
				my $name2 = uc($list[$j]);
				if( $name2 eq 'SERVICE' ) {
					my %attrs = upperKeys( %{ shift @{$list[$j+1]} } );
					my @filters = @{$list[$j+1]};

					my $service = $attrs{'NAME'};

					%{ $this->{services}{$service} } = (
						'DESCRIPTION' => $attrs{'DESCRIPTION'},
						'FILTERS' => ()
						);

					for( my $k=0; $k<=$#filters; $k+=2 ) {
						my $name3 = uc( $filters[$k] );
						my %filter = upperKeys( %{shift @{$filters[$k+1]}} );
						if( $name3 eq 'FILTER' ) {
							%{$this->{services}{$service}{FILTERS}[$#{$this->{services}{$service}{FILTERS}}+1]} = %filter;
						}
					}
				}
			}
		}
	}
}


sub upperKeys {
	my %hash = @_;
	my %newHash;
	@ks = keys %hash;
	foreach $k (@ks) {
		$newHash{uc($k)} = $hash{$k};
	}
	return %newHash;
}

sub SaveFirewall {
	my $this = shift;
	$this->SaveFirewallAs( $this->{fw_file} );
}

sub SaveFirewallAs {
	my $this = shift;
	my ($fwFile) = @_;

	my %fw = %{ $this->{fw} };

	my $xml = "<firewall>\n";
	$xml .= "\n";
	foreach my $k (keys %{$fw{'ZONE'}}) {
		if( $k ne 'FIREWALL' ) {
			$xml .= $this->attr2xml( 'zone', %{$fw{'ZONE'}{$k}} );
		}
	}
	$xml .= "\n";
	foreach my $k (keys %{$fw{'NET'}}) {
		$xml .= $this->attr2xml( 'net', %{$fw{'NET'}{$k}} );
	}
	$xml .= "\n";
	foreach my $k (keys %{$fw{'HOST'}}) {
		$xml .= $this->attr2xml( 'host', %{$fw{'HOST'}{$k}} );
	}
	my @redir = @{$fw{'REDIR'}};
	for my $i (0..$#redir) {
		$xml .= $this->attr2xml( 'redir', %{$redir[$i]} );
	}
	$xml .= "\n";
	foreach my $k (keys %{$fw{'GROUP'}}) {
		$xml .= "<group name=\"$k\">\n";
		foreach my $item (@{$fw{'GROUP'}{$k}}) {
			$xml .= "\t<item name=\"$item\"/>\n";
		}
		$xml .= "</group>\n";
	}
	$xml .= "\n";
	my @nats = @{$fw{'NAT'}};
	for my $i (0..$#nats) {
		$xml .= $this->attr2xml( 'nat', %{$nats[$i]} );
	}
	$xml .= "\n";
	my @masq = @{$fw{'MASQUERADE'}};
	for my $i (0..$#masq) {
		$xml .= $this->attr2xml( 'masquerade', %{$masq[$i]} );
	}
	$xml .= "\n";
	my @rules = @{$fw{'RULE'}};
	for my $i (0..$#rules) {
		$xml .= $this->attr2xml( 'rule', %{$rules[$i]} );
	}
	$xml .= "\n";
	$xml .= "</firewall>\n";

	open( FWFILE, ">$fwFile" );
	print FWFILE $xml;
	close( FWFILE );
}
sub attr2xml {
	my $this = shift;
	my ($tag, %attr, @order) = @_;
	my $appo = "<$tag";
	foreach my $k (keys %attr) {
		$appo .= ' '.lc($k).'="'.$attr{$k}.'"';
	}
	$appo .= "/>\n";
	return $appo;
}




sub expand_item {
	$this = shift;
	$item = shift;
	$itemType = $fwItems{$item};

	my $zone = '';
	my $ip = '';

	if( $itemType eq 'ZONE' ) {
		$zone = $item;
		$ip = '0.0.0.0/0';
	}
	if( $itemType eq 'NET' ) {
		$zone = $fw{NET}{$item}{ZONE};
		$ip = $fw{NET}{$item}{IP}.'/'.$fw{NET}{$item}{NETMASK};
	}
	if( $itemType eq 'HOST' ) {
		$zone = $fw{HOST}{$item}{ZONE};
		$ip = $fw{HOST}{$item}{IP}.'/32';
	}
	return ($zone, $ip);
}


1;

__END__








